@@ -401,7 +401,9 @@ class ObserverSpanProcessor(SpanProcessor):
                         )
                         delete_ids = Trace.query.order_by("start_time")[
                             :excess_count
-                        ].values_list("id", flat=True)
+                        ].values_list(  # type: ignore[union-attr]
+                            "id", flat=True
+                        )
                         Trace.query.filter(id__in=delete_ids).delete()
                 except Exception as e:
                     logger.warning(
